@@ -7,15 +7,16 @@ use crate::{
     civil,
     duration::{Duration, SDuration},
     error::{err, Error, ErrorContext},
+    shared::util::itime::IOffset,
     span::Span,
     timestamp::Timestamp,
     tz::{AmbiguousOffset, AmbiguousTimestamp, AmbiguousZoned, TimeZone},
     util::{
         array_str::ArrayStr,
-        rangeint::{RFrom, RInto, TryRFrom},
+        rangeint::{self, Composite, RFrom, RInto, TryRFrom},
         t::{self, C},
     },
-    SignedDuration,
+    RoundMode, SignedDuration, SignedDurationRound, Unit,
 };
 
 /// An enum indicating whether a particular datetime  is in DST or not.
@@ -231,7 +232,7 @@ impl Offset {
     // warrant its existence. And I think I'd rather `Offset::hms` be const and
     // exported instead of this monstrosity.
     #[inline]
-    const fn constant_seconds(seconds: i32) -> Offset {
+    pub(crate) const fn constant_seconds(seconds: i32) -> Offset {
         if !t::SpanZoneOffset::contains(seconds) {
             panic!("invalid time zone offset seconds")
         }
@@ -376,7 +377,7 @@ impl Offset {
     /// assert!(!tz::offset(-5).is_positive());
     /// ```
     pub fn is_positive(self) -> bool {
-        self.seconds_ranged() > 0
+        self.seconds_ranged() > C(0)
     }
 
     /// Returns true if and only if this offset is less than zero.
@@ -391,7 +392,7 @@ impl Offset {
     /// assert!(tz::offset(-5).is_negative());
     /// ```
     pub fn is_negative(self) -> bool {
-        self.seconds_ranged() < 0
+        self.seconds_ranged() < C(0)
     }
 
     /// Returns true if and only if this offset is zero.
@@ -408,7 +409,7 @@ impl Offset {
     /// assert!(!tz::offset(-5).is_zero());
     /// ```
     pub fn is_zero(self) -> bool {
-        self.seconds_ranged() == 0
+        self.seconds_ranged() == C(0)
     }
 
     /// Converts this offset into a [`TimeZone`].
@@ -445,7 +446,21 @@ impl Offset {
     /// ```
     #[inline]
     pub fn to_datetime(self, timestamp: Timestamp) -> civil::DateTime {
-        timestamp_to_datetime_zulu(timestamp, self)
+        let idt = timestamp.to_itimestamp().zip2(self.to_ioffset()).map(
+            #[allow(unused_mut)]
+            |(mut its, ioff)| {
+                // This is tricky, but if we have a minimal number of seconds,
+                // then the minimum possible nanosecond value is actually 0.
+                // So we clamp it in this case. (This encodes the invariant
+                // enforced by `Timestamp::new`.)
+                #[cfg(debug_assertions)]
+                if its.second == t::UnixSeconds::MIN_REPR {
+                    its.nanosecond = 0;
+                }
+                its.to_datetime(ioff)
+            },
+        );
+        civil::DateTime::from_idatetime(idt)
     }
 
     /// Converts the given civil datetime to a timestamp using this offset.
@@ -507,7 +522,16 @@ impl Offset {
         self,
         dt: civil::DateTime,
     ) -> Result<Timestamp, Error> {
-        datetime_zulu_to_timestamp(dt, self)
+        let its = dt
+            .to_idatetime()
+            .zip2(self.to_ioffset())
+            .map(|(idt, ioff)| idt.to_timestamp(ioff));
+        Timestamp::from_itimestamp(its).with_context(|| {
+            err!(
+                "converting {dt} with offset {offset} to timestamp overflowed",
+                offset = self,
+            )
+        })
     }
 
     /// Adds the given span of time to this offset.
@@ -786,12 +810,12 @@ impl Offset {
     ///
     /// assert_eq!(
     ///     tz::offset(-5).until(tz::Offset::UTC),
-    ///     (5 * 60 * 60).seconds(),
+    ///     (5 * 60 * 60).seconds().fieldwise(),
     /// );
     /// // Flipping the operands in this case results in a negative span.
     /// assert_eq!(
     ///     tz::Offset::UTC.until(tz::offset(-5)),
-    ///     -(5 * 60 * 60).seconds(),
+    ///     -(5 * 60 * 60).seconds().fieldwise(),
     /// );
     /// ```
     #[inline]
@@ -817,12 +841,12 @@ impl Offset {
     ///
     /// assert_eq!(
     ///     tz::Offset::UTC.since(tz::offset(-5)),
-    ///     (5 * 60 * 60).seconds(),
+    ///     (5 * 60 * 60).seconds().fieldwise(),
     /// );
     /// // Flipping the operands in this case results in a negative span.
     /// assert_eq!(
     ///     tz::offset(-5).since(tz::Offset::UTC),
-    ///     -(5 * 60 * 60).seconds(),
+    ///     -(5 * 60 * 60).seconds().fieldwise(),
     /// );
     /// ```
     #[inline]
@@ -888,10 +912,80 @@ impl Offset {
         SignedDuration::offset_until(other, self)
     }
 
-    /// Returns this offset as a [`SignedDuration`].
+    /// Returns a new offset that is rounded according to the given
+    /// configuration.
+    ///
+    /// Rounding an offset has a number of parameters, all of which are
+    /// optional. When no parameters are given, then no rounding is done, and
+    /// the offset as given is returned. That is, it's a no-op.
+    ///
+    /// As is consistent with `Offset` itself, rounding only supports units of
+    /// hours, minutes or seconds. If any other unit is provided, then an error
+    /// is returned.
+    ///
+    /// The parameters are, in brief:
+    ///
+    /// * [`OffsetRound::smallest`] sets the smallest [`Unit`] that is allowed
+    /// to be non-zero in the offset returned. By default, it is set to
+    /// [`Unit::Second`], i.e., no rounding occurs. When the smallest unit is
+    /// set to something bigger than seconds, then the non-zero units in the
+    /// offset smaller than the smallest unit are used to determine how the
+    /// offset should be rounded. For example, rounding `+01:59` to the nearest
+    /// hour using the default rounding mode would produce `+02:00`.
+    /// * [`OffsetRound::mode`] determines how to handle the remainder
+    /// when rounding. The default is [`RoundMode::HalfExpand`], which
+    /// corresponds to how you were likely taught to round in school.
+    /// Alternative modes, like [`RoundMode::Trunc`], exist too. For example,
+    /// a truncating rounding of `+01:59` to the nearest hour would
+    /// produce `+01:00`.
+    /// * [`OffsetRound::increment`] sets the rounding granularity to
+    /// use for the configured smallest unit. For example, if the smallest unit
+    /// is minutes and the increment is `15`, then the offset returned will
+    /// always have its minute component set to a multiple of `15`.
+    ///
+    /// # Errors
+    ///
+    /// In general, there are two main ways for rounding to fail: an improper
+    /// configuration like trying to round an offset to the nearest unit other
+    /// than hours/minutes/seconds, or when overflow occurs. Overflow can occur
+    /// when the offset would exceed the minimum or maximum `Offset` values.
+    /// Typically, this can only realistically happen if the offset before
+    /// rounding is already close to its minimum or maximum value.
+    ///
+    /// # Example: rounding to the nearest multiple of 15 minutes
+    ///
+    /// Most time zone offsets fall on an hour boundary, but some fall on the
+    /// half-hour or even 15 minute boundary:
+    ///
+    /// ```
+    /// use jiff::{tz::Offset, Unit};
+    ///
+    /// let offset = Offset::from_seconds(-(44 * 60 + 30)).unwrap();
+    /// let rounded = offset.round((Unit::Minute, 15))?;
+    /// assert_eq!(rounded, Offset::from_seconds(-45 * 60).unwrap());
+    ///
+    /// # Ok::<(), Box<dyn std::error::Error>>(())
+    /// ```
+    ///
+    /// # Example: rounding can fail via overflow
+    ///
+    /// ```
+    /// use jiff::{tz::Offset, Unit};
+    ///
+    /// assert_eq!(Offset::MAX.to_string(), "+25:59:59");
+    /// assert_eq!(
+    ///     Offset::MAX.round(Unit::Minute).unwrap_err().to_string(),
+    ///     "rounding offset `+25:59:59` resulted in a duration of 26h, \
+    ///      which overflows `Offset`",
+    /// );
+    /// ```
     #[inline]
-    pub(crate) fn to_duration(self) -> SignedDuration {
-        SignedDuration::from_secs(i64::from(self.seconds()))
+    pub fn round<R: Into<OffsetRound>>(
+        self,
+        options: R,
+    ) -> Result<Offset, Error> {
+        let options: OffsetRound = options.into();
+        options.round(self)
     }
 }
 
@@ -941,8 +1035,42 @@ impl Offset {
         Offset { span: seconds.rinto() }
     }
 
+    /*
     #[inline]
-    pub(crate) fn seconds_ranged(self) -> t::SpanZoneOffset {
+    pub(crate) fn from_ioffset(ioff: Composite<IOffset>) -> Offset {
+        let span = rangeint::uncomposite!(ioff, c => (c.second));
+        Offset { span: span.to_rint() }
+    }
+    */
+
+    #[inline]
+    pub(crate) fn to_ioffset(self) -> Composite<IOffset> {
+        rangeint::composite! {
+            (second = self.span) => {
+                IOffset { second }
+            }
+        }
+    }
+
+    #[inline]
+    pub(crate) const fn from_ioffset_const(ioff: IOffset) -> Offset {
+        Offset::from_seconds_unchecked(ioff.second)
+    }
+
+    #[inline]
+    pub(crate) const fn from_seconds_unchecked(second: i32) -> Offset {
+        Offset { span: t::SpanZoneOffset::new_unchecked(second) }
+    }
+
+    /*
+    #[inline]
+    pub(crate) const fn to_ioffset_const(self) -> IOffset {
+        IOffset { second: self.span.get_unchecked() }
+    }
+    */
+
+    #[inline]
+    pub(crate) const fn seconds_ranged(self) -> t::SpanZoneOffset {
         self.span
     }
 
@@ -979,7 +1107,7 @@ impl Offset {
 
 impl core::fmt::Debug for Offset {
     fn fmt(&self, f: &mut core::fmt::Formatter) -> core::fmt::Result {
-        let sign = if self.seconds_ranged() < 0 { "-" } else { "" };
+        let sign = if self.seconds_ranged() < C(0) { "-" } else { "" };
         write!(
             f,
             "{sign}{:02}:{:02}:{:02}",
@@ -992,7 +1120,7 @@ impl core::fmt::Debug for Offset {
 
 impl core::fmt::Display for Offset {
     fn fmt(&self, f: &mut core::fmt::Formatter) -> core::fmt::Result {
-        let sign = if self.span < 0 { "-" } else { "+" };
+        let sign = if self.span < C(0) { "-" } else { "+" };
         let hours = self.part_hours_ranged().abs().get();
         let minutes = self.part_minutes_ranged().abs().get();
         let seconds = self.part_seconds_ranged().abs().get();
@@ -1182,6 +1310,51 @@ impl Neg for Offset {
     }
 }
 
+/// Converts a `SignedDuration` to a time zone offset.
+///
+/// If the signed duration has fractional seconds, then it is automatically
+/// rounded to the nearest second. (Because an `Offset` has only second
+/// precision.)
+///
+/// # Errors
+///
+/// This returns an error if the duration overflows the limits of an `Offset`.
+///
+/// # Example
+///
+/// ```
+/// use jiff::{tz::{self, Offset}, SignedDuration};
+///
+/// let sdur = SignedDuration::from_secs(-5 * 60 * 60);
+/// let offset = Offset::try_from(sdur)?;
+/// assert_eq!(offset, tz::offset(-5));
+///
+/// // Sub-seconds results in rounded.
+/// let sdur = SignedDuration::new(-5 * 60 * 60, -500_000_000);
+/// let offset = Offset::try_from(sdur)?;
+/// assert_eq!(offset, tz::Offset::from_seconds(-(5 * 60 * 60 + 1)).unwrap());
+///
+/// # Ok::<(), Box<dyn std::error::Error>>(())
+/// ```
+impl TryFrom<SignedDuration> for Offset {
+    type Error = Error;
+
+    fn try_from(sdur: SignedDuration) -> Result<Offset, Error> {
+        let mut seconds = sdur.as_secs();
+        let subsec = sdur.subsec_nanos();
+        if subsec >= 500_000_000 {
+            seconds = seconds.saturating_add(1);
+        } else if subsec <= -500_000_000 {
+            seconds = seconds.saturating_sub(1);
+        }
+        let seconds = i32::try_from(seconds).map_err(|_| {
+            err!("`SignedDuration` of {sdur} overflows `Offset`")
+        })?;
+        Offset::from_seconds(seconds)
+            .map_err(|_| err!("`SignedDuration` of {sdur} overflows `Offset`"))
+    }
+}
+
 /// Options for [`Offset::checked_add`] and [`Offset::checked_sub`].
 ///
 /// This type provides a way to ergonomically add one of a few different
@@ -1276,6 +1449,191 @@ impl<'a> From<&'a UnsignedDuration> for OffsetArithmetic {
     }
 }
 
+/// Options for [`Offset::round`].
+///
+/// This type provides a way to configure the rounding of an offset. This
+/// includes setting the smallest unit (i.e., the unit to round), the rounding
+/// increment and the rounding mode (e.g., "ceil" or "truncate").
+///
+/// [`Offset::round`] accepts anything that implements
+/// `Into<OffsetRound>`. There are a few key trait implementations that
+/// make this convenient:
+///
+/// * `From<Unit> for OffsetRound` will construct a rounding
+/// configuration where the smallest unit is set to the one given.
+/// * `From<(Unit, i64)> for OffsetRound` will construct a rounding
+/// configuration where the smallest unit and the rounding increment are set to
+/// the ones given.
+///
+/// In order to set other options (like the rounding mode), one must explicitly
+/// create a `OffsetRound` and pass it to `Offset::round`.
+///
+/// # Example
+///
+/// This example shows how to always round up to the nearest half-hour:
+///
+/// ```
+/// use jiff::{tz::{Offset, OffsetRound}, RoundMode, Unit};
+///
+/// let offset = Offset::from_seconds(4 * 60 * 60 + 17 * 60).unwrap();
+/// let rounded = offset.round(
+///     OffsetRound::new()
+///         .smallest(Unit::Minute)
+///         .increment(30)
+///         .mode(RoundMode::Expand),
+/// )?;
+/// assert_eq!(rounded, Offset::from_seconds(4 * 60 * 60 + 30 * 60).unwrap());
+///
+/// # Ok::<(), Box<dyn std::error::Error>>(())
+/// ```
+#[derive(Clone, Copy, Debug)]
+pub struct OffsetRound(SignedDurationRound);
+
+impl OffsetRound {
+    /// Create a new default configuration for rounding a time zone offset via
+    /// [`Offset::round`].
+    ///
+    /// The default configuration does no rounding.
+    #[inline]
+    pub fn new() -> OffsetRound {
+        OffsetRound(SignedDurationRound::new().smallest(Unit::Second))
+    }
+
+    /// Set the smallest units allowed in the offset returned. These are the
+    /// units that the offset is rounded to.
+    ///
+    /// # Errors
+    ///
+    /// The unit must be [`Unit::Hour`], [`Unit::Minute`] or [`Unit::Second`].
+    ///
+    /// # Example
+    ///
+    /// A basic example that rounds to the nearest minute:
+    ///
+    /// ```
+    /// use jiff::{tz::Offset, Unit};
+    ///
+    /// let offset = Offset::from_seconds(-(5 * 60 * 60 + 30)).unwrap();
+    /// assert_eq!(offset.round(Unit::Hour)?, Offset::from_hours(-5).unwrap());
+    ///
+    /// # Ok::<(), Box<dyn std::error::Error>>(())
+    /// ```
+    #[inline]
+    pub fn smallest(self, unit: Unit) -> OffsetRound {
+        OffsetRound(self.0.smallest(unit))
+    }
+
+    /// Set the rounding mode.
+    ///
+    /// This defaults to [`RoundMode::HalfExpand`], which makes rounding work
+    /// like how you were taught in school.
+    ///
+    /// # Example
+    ///
+    /// A basic example that rounds to the nearest hour, but changing its
+    /// rounding mode to truncation:
+    ///
+    /// ```
+    /// use jiff::{tz::{Offset, OffsetRound}, RoundMode, Unit};
+    ///
+    /// let offset = Offset::from_seconds(-(5 * 60 * 60 + 30 * 60)).unwrap();
+    /// assert_eq!(
+    ///     offset.round(OffsetRound::new()
+    ///         .smallest(Unit::Hour)
+    ///         .mode(RoundMode::Trunc),
+    ///     )?,
+    ///     // The default round mode does rounding like
+    ///     // how you probably learned in school, and would
+    ///     // result in rounding to -6 hours. But we
+    ///     // change it to truncation here, which makes it
+    ///     // round -5.
+    ///     Offset::from_hours(-5).unwrap(),
+    /// );
+    ///
+    /// # Ok::<(), Box<dyn std::error::Error>>(())
+    /// ```
+    #[inline]
+    pub fn mode(self, mode: RoundMode) -> OffsetRound {
+        OffsetRound(self.0.mode(mode))
+    }
+
+    /// Set the rounding increment for the smallest unit.
+    ///
+    /// The default value is `1`. Other values permit rounding the smallest
+    /// unit to the nearest integer increment specified. For example, if the
+    /// smallest unit is set to [`Unit::Minute`], then a rounding increment of
+    /// `30` would result in rounding in increments of a half hour. That is,
+    /// the only minute value that could result would be `0` or `30`.
+    ///
+    /// # Errors
+    ///
+    /// The rounding increment must divide evenly into the next highest unit
+    /// after the smallest unit configured (and must not be equivalent to
+    /// it). For example, if the smallest unit is [`Unit::Second`], then
+    /// *some* of the valid values for the rounding increment are `1`, `2`,
+    /// `4`, `5`, `15` and `30`. Namely, any integer that divides evenly into
+    /// `60` seconds since there are `60` seconds in the next highest unit
+    /// (minutes).
+    ///
+    /// # Example
+    ///
+    /// This shows how to round an offset to the nearest 30 minute increment:
+    ///
+    /// ```
+    /// use jiff::{tz::Offset, Unit};
+    ///
+    /// let offset = Offset::from_seconds(4 * 60 * 60 + 15 * 60).unwrap();
+    /// assert_eq!(
+    ///     offset.round((Unit::Minute, 30))?,
+    ///     Offset::from_seconds(4 * 60 * 60 + 30 * 60).unwrap(),
+    /// );
+    ///
+    /// # Ok::<(), Box<dyn std::error::Error>>(())
+    /// ```
+    #[inline]
+    pub fn increment(self, increment: i64) -> OffsetRound {
+        OffsetRound(self.0.increment(increment))
+    }
+
+    /// Does the actual offset rounding.
+    fn round(&self, offset: Offset) -> Result<Offset, Error> {
+        let smallest = self.0.get_smallest();
+        if !(Unit::Second <= smallest && smallest <= Unit::Hour) {
+            return Err(err!(
+                "rounding `Offset` failed because \
+                 a unit of {plural} was provided, but offset rounding \
+                 can only use hours, minutes or seconds",
+                plural = smallest.plural(),
+            ));
+        }
+        let rounded_sdur = SignedDuration::from(offset).round(self.0)?;
+        Offset::try_from(rounded_sdur).map_err(|_| {
+            err!(
+                "rounding offset `{offset}` resulted in a duration \
+                 of {rounded_sdur:?}, which overflows `Offset`",
+            )
+        })
+    }
+}
+
+impl Default for OffsetRound {
+    fn default() -> OffsetRound {
+        OffsetRound::new()
+    }
+}
+
+impl From<Unit> for OffsetRound {
+    fn from(unit: Unit) -> OffsetRound {
+        OffsetRound::default().smallest(unit)
+    }
+}
+
+impl From<(Unit, i64)> for OffsetRound {
+    fn from((unit, increment): (Unit, i64)) -> OffsetRound {
+        OffsetRound::default().smallest(unit).increment(increment)
+    }
+}
+
 /// Configuration for resolving disparities between an offset and a time zone.
 ///
 /// A conflict between an offset and a time zone most commonly appears in a
@@ -1473,6 +1831,125 @@ impl OffsetConflict {
         offset: Offset,
         tz: TimeZone,
     ) -> Result<AmbiguousZoned, Error> {
+        self.resolve_with(dt, offset, tz, |off1, off2| off1 == off2)
+    }
+
+    /// Resolve a potential conflict between an [`Offset`] and a [`TimeZone`]
+    /// using the given definition of equality for an `Offset`.
+    ///
+    /// The equality predicate is always given a pair of offsets where the
+    /// first is the offset given to `resolve_with` and the second is the
+    /// offset found in the `TimeZone`.
+    ///
+    /// # Errors
+    ///
+    /// This returns an error if this would have returned a timestamp outside
+    /// of its minimum and maximum values.
+    ///
+    /// This can also return an error when using the [`OffsetConflict::Reject`]
+    /// strategy. Namely, when using the `Reject` strategy, any offset that is
+    /// not compatible with the given datetime and time zone will always result
+    /// in an error.
+    ///
+    /// # Example
+    ///
+    /// Unlike [`OffsetConflict::resolve`], this routine permits overriding
+    /// the definition of equality used for comparing offsets. In
+    /// `OffsetConflict::resolve`, exact equality is used. This can be
+    /// troublesome in some cases when a time zone has an offset with
+    /// fractional minutes, such as `Africa/Monrovia` before 1972.
+    ///
+    /// Because RFC 3339 and RFC 9557 do not support time zone offsets
+    /// with fractional minutes, Jiff will serialize offsets with
+    /// fractional minutes by rounding to the nearest minute. This
+    /// will result in a different offset than what is actually
+    /// used in the time zone. Parsing this _should_ succeed, but
+    /// if exact offset equality is used, it won't. This is why a
+    /// [`fmt::temporal::DateTimeParser`](crate::fmt::temporal::DateTimeParser)
+    /// uses this routine with offset equality that rounds offsets to the
+    /// nearest minute before comparison.
+    ///
+    /// ```
+    /// use jiff::{civil::date, tz::{Offset, OffsetConflict, TimeZone}, Unit};
+    ///
+    /// let dt = date(1968, 2, 1).at(23, 15, 0, 0);
+    /// let offset = Offset::from_seconds(-(44 * 60 + 30)).unwrap();
+    /// let zdt = dt.in_tz("Africa/Monrovia")?;
+    /// assert_eq!(zdt.offset(), offset);
+    /// // Notice that the offset has been rounded!
+    /// assert_eq!(zdt.to_string(), "1968-02-01T23:15:00-00:45[Africa/Monrovia]");
+    ///
+    /// // Now imagine parsing extracts the civil datetime, the offset and
+    /// // the time zone, and then naively does exact offset comparison:
+    /// let tz = TimeZone::get("Africa/Monrovia")?;
+    /// // This is the parsed offset, which won't precisely match the actual
+    /// // offset used by `Africa/Monrovia` at this time.
+    /// let offset = Offset::from_seconds(-45 * 60).unwrap();
+    /// let result = OffsetConflict::Reject.resolve(dt, offset, tz.clone());
+    /// assert_eq!(
+    ///     result.unwrap_err().to_string(),
+    ///     "datetime 1968-02-01T23:15:00 could not resolve to a timestamp \
+    ///      since 'reject' conflict resolution was chosen, and because \
+    ///      datetime has offset -00:45, but the time zone Africa/Monrovia \
+    ///      for the given datetime unambiguously has offset -00:44:30",
+    /// );
+    /// let is_equal = |parsed: Offset, candidate: Offset| {
+    ///     parsed == candidate || candidate.round(Unit::Minute).map_or(
+    ///         parsed == candidate,
+    ///         |candidate| parsed == candidate,
+    ///     )
+    /// };
+    /// let zdt = OffsetConflict::Reject.resolve_with(
+    ///     dt,
+    ///     offset,
+    ///     tz.clone(),
+    ///     is_equal,
+    /// )?.unambiguous()?;
+    /// // Notice that the offset is the actual offset from the time zone:
+    /// assert_eq!(zdt.offset(), Offset::from_seconds(-(44 * 60 + 30)).unwrap());
+    /// // But when we serialize, the offset gets rounded. If we didn't
+    /// // do this, we'd risk the datetime not being parsable by other
+    /// // implementations since RFC 3339 and RFC 9557 don't support fractional
+    /// // minutes in the offset.
+    /// assert_eq!(zdt.to_string(), "1968-02-01T23:15:00-00:45[Africa/Monrovia]");
+    ///
+    /// # Ok::<(), Box<dyn std::error::Error>>(())
+    /// ```
+    ///
+    /// And indeed, notice that parsing uses this same kind of offset equality
+    /// to permit zoned datetimes whose offsets would be equivalent after
+    /// rounding:
+    ///
+    /// ```
+    /// use jiff::{tz::Offset, Zoned};
+    ///
+    /// let zdt: Zoned = "1968-02-01T23:15:00-00:45[Africa/Monrovia]".parse()?;
+    /// // As above, notice that even though we parsed `-00:45` as the
+    /// // offset, the actual offset of our zoned datetime is the correct
+    /// // one from the time zone.
+    /// assert_eq!(zdt.offset(), Offset::from_seconds(-(44 * 60 + 30)).unwrap());
+    /// // And similarly, re-serializing it results in rounding the offset
+    /// // again for compatibility with RFC 3339 and RFC 9557.
+    /// assert_eq!(zdt.to_string(), "1968-02-01T23:15:00-00:45[Africa/Monrovia]");
+    ///
+    /// // And we also support parsing the actual fractional minute offset
+    /// // as well:
+    /// let zdt: Zoned = "1968-02-01T23:15:00-00:44:30[Africa/Monrovia]".parse()?;
+    /// assert_eq!(zdt.offset(), Offset::from_seconds(-(44 * 60 + 30)).unwrap());
+    /// assert_eq!(zdt.to_string(), "1968-02-01T23:15:00-00:45[Africa/Monrovia]");
+    ///
+    /// # Ok::<(), Box<dyn std::error::Error>>(())
+    /// ```
+    pub fn resolve_with<F>(
+        self,
+        dt: civil::DateTime,
+        offset: Offset,
+        tz: TimeZone,
+        is_equal: F,
+    ) -> Result<AmbiguousZoned, Error>
+    where
+        F: FnMut(Offset, Offset) -> bool,
+    {
         match self {
             // In this case, we ignore any TZ annotation (although still
             // require that it exists) and always use the provided offset.
@@ -1485,13 +1962,13 @@ impl OffsetConflict {
             OffsetConflict::AlwaysTimeZone => Ok(tz.into_ambiguous_zoned(dt)),
             // In this case, we use the offset if it's correct, but otherwise
             // fall back to the time zone annotation if it's not.
-            OffsetConflict::PreferOffset => {
-                Ok(OffsetConflict::resolve_via_prefer(dt, offset, tz))
-            }
+            OffsetConflict::PreferOffset => Ok(
+                OffsetConflict::resolve_via_prefer(dt, offset, tz, is_equal),
+            ),
             // In this case, if the offset isn't possible for the provided time
             // zone annotation, then we return an error.
             OffsetConflict::Reject => {
-                OffsetConflict::resolve_via_reject(dt, offset, tz)
+                OffsetConflict::resolve_via_reject(dt, offset, tz, is_equal)
             }
         }
     }
@@ -1509,18 +1986,18 @@ impl OffsetConflict {
         dt: civil::DateTime,
         given: Offset,
         tz: TimeZone,
+        mut is_equal: impl FnMut(Offset, Offset) -> bool,
     ) -> AmbiguousZoned {
         use crate::tz::AmbiguousOffset::*;
 
         let amb = tz.to_ambiguous_timestamp(dt);
         match amb.offset() {
-            // Basically, if the offset parsed matches one of our ambiguous
-            // offsets, then the offset is used to resolve the ambiguity and
-            // yields an unambiguous result. But in every other case, the
-            // offset parsed is completely ignored. (And this may result in
-            // returning an ambiguous instant.)
-            Gap { before, after } | Fold { before, after }
-                if given == before || given == after =>
+            // We only look for folds because we consider all offsets for gaps
+            // to be invalid. Which is consistent with how they're treated as
+            // `OffsetConflict::Reject`. Thus, like any other invalid offset,
+            // we fallback to disambiguation (which is handled by the caller).
+            Fold { before, after }
+                if is_equal(given, before) || is_equal(given, after) =>
             {
                 let kind = Unambiguous { offset: given };
                 AmbiguousTimestamp::new(dt, kind)
@@ -1546,12 +2023,13 @@ impl OffsetConflict {
         dt: civil::DateTime,
         given: Offset,
         tz: TimeZone,
+        mut is_equal: impl FnMut(Offset, Offset) -> bool,
     ) -> Result<AmbiguousZoned, Error> {
         use crate::tz::AmbiguousOffset::*;
 
         let amb = tz.to_ambiguous_timestamp(dt);
         match amb.offset() {
-            Unambiguous { offset } if given != offset => Err(err!(
+            Unambiguous { offset } if !is_equal(given, offset) => Err(err!(
                 "datetime {dt} could not resolve to a timestamp since \
                  'reject' conflict resolution was chosen, and because \
                  datetime has offset {given}, but the time zone {tzname} for \
@@ -1559,32 +2037,15 @@ impl OffsetConflict {
                 tzname = tz.diagnostic_name(),
             )),
             Unambiguous { .. } => Ok(amb.into_ambiguous_zoned(tz)),
-            Gap { before, after } if given != before && given != after => {
-                // Temporal actually seems to report an error whenever a gap
-                // is found, even if the parsed offset matches one of the two
-                // offsets in the gap. I think the reasoning is because the
-                // parsed offset and offsets from the gap will flip-flop, thus
-                // resulting in a datetime with an offset distinct from the one
-                // given. Here's an example. Consider this datetime:
-                //
-                //     2024-03-10T02:30-04:00[America/New_York]
-                //
-                // This occurs in the EST->EDT transition gap, such that
-                // `02:30` never actually appears on a clock for folks in
-                // `America/New_York`. The `-04` offset means that the
-                // timestamp this corresponds to is unambiguous. Namely, it is:
-                //
-                //     2024-03-10T06:30Z
-                //
-                // This instant, when converted to `America/New_York`, is:
-                //
-                //     2024-03-10T01:30-05:00[America/New_York]
-                //
-                // As you can see, the offset flip-flopped. The same flip-flop
-                // happens the other way if you use `02:30-05:00`. Presumably,
-                // Temporal errors here because the offset changes. But the
-                // instant in time is the same *and* it is unambiguous. So it's
-                // not clear to me why we ought to error in this case.
+            Gap { before, after } => {
+                // In `jiff 0.1`, we reported an error when we found a gap
+                // where neither offset matched what was given. But now we
+                // report an error whenever we find a gap, as we consider
+                // all offsets to be invalid for the gap. This now matches
+                // Temporal's behavior which I think is more consistent. And in
+                // particular, this makes it more consistent with the behavior
+                // of `PreferOffset` when a gap is found (which was also
+                // changed to treat all offsets in a gap as invalid).
                 //
                 // Ref: https://github.com/tc39/proposal-temporal/issues/2892
                 Err(err!(
@@ -1592,12 +2053,14 @@ impl OffsetConflict {
                      since 'reject' conflict resolution was chosen, and \
                      because datetime has offset {given}, but the time \
                      zone {tzname} for the given datetime falls in a gap \
-                     between offsets {before} and {after}, neither of which \
-                     match the offset",
+                     (between offsets {before} and {after}), and all \
+                     offsets for a gap are regarded as invalid",
                     tzname = tz.diagnostic_name(),
                 ))
             }
-            Fold { before, after } if given != before && given != after => {
+            Fold { before, after }
+                if !is_equal(given, before) && !is_equal(given, after) =>
+            {
                 Err(err!(
                     "datetime {dt} could not resolve to timestamp \
                      since 'reject' conflict resolution was chosen, and \
@@ -1608,107 +2071,10 @@ impl OffsetConflict {
                     tzname = tz.diagnostic_name(),
                 ))
             }
-            Gap { .. } | Fold { .. } => {
+            Fold { .. } => {
                 let kind = Unambiguous { offset: given };
                 Ok(AmbiguousTimestamp::new(dt, kind).into_ambiguous_zoned(tz))
             }
         }
     }
 }
-
-fn timestamp_to_datetime_zulu(
-    timestamp: Timestamp,
-    offset: Offset,
-) -> civil::DateTime {
-    let mut second = timestamp.as_second_ranged();
-    let nanosecond = timestamp.subsec_nanosecond_ranged();
-    second += offset.seconds_ranged();
-    // This is a little tricky. The day, second and nanosecond can all be
-    // shifted when nanoseconds are themselves negative. Since all three
-    // can vary, we need to shove them into a single `NoUnits::vary_many`
-    // closure to make range arithmetic work correctly. We convert them
-    // back to their proper units afterwards.
-    let day = second.without_bounds().div_floor(t::SECONDS_PER_CIVIL_DAY);
-    let second: t::NoUnits =
-        second.without_bounds().rem_floor(t::SECONDS_PER_CIVIL_DAY);
-    let nanosecond = nanosecond.without_bounds();
-    let [delta_day, delta_second, delta_nanosecond] = t::NoUnits::vary_many(
-        [day, second, nanosecond],
-        |[_, second, nanosecond]| {
-            if nanosecond >= 0 {
-                [C(0), C(0), C(0)]
-            } else if second > 0 {
-                [C(0), C(-1), t::NANOS_PER_SECOND.rinto()]
-            } else {
-                [
-                    C(-1),
-                    t::SECONDS_PER_CIVIL_DAY - C(1),
-                    t::NANOS_PER_SECOND.rinto(),
-                ]
-            }
-        },
-    );
-
-    // Given the ranges on UnixSeconds and FractionalNanoseconds, it is
-    // technically possible that the subtraction on `day` here could
-    // overflow. But! This can only happen when the Unix second is its
-    // minimal value, *and* the number of nanoseconds is non-zero. That
-    // case is specifically rejected when constructing a `Timestamp` value,
-    // so it is correct to assert that the overflow can never happen.
-    let day = t::UnixEpochDays::rfrom(day)
-        .try_checked_add("day", delta_day)
-        .unwrap();
-    let second = (second + delta_second) * t::NANOS_PER_SECOND;
-    let nanosecond = nanosecond + delta_nanosecond;
-    let civil_day_nanosecond = second + nanosecond;
-    let date = civil::Date::from_unix_epoch_days(day);
-    let time = civil::Time::from_nanosecond(civil_day_nanosecond);
-    civil::DateTime::from_parts(date, time)
-}
-
-fn datetime_zulu_to_timestamp(
-    dt: civil::DateTime,
-    offset: Offset,
-) -> Result<Timestamp, Error> {
-    let (date, time) = (dt.date(), dt.time());
-    let day = date.to_unix_epoch_days().without_bounds();
-    let civil_day_nanosecond = time.to_nanosecond().without_bounds();
-    let second = civil_day_nanosecond.div_floor(t::NANOS_PER_SECOND);
-    let nanosecond = civil_day_nanosecond.rem_floor(t::NANOS_PER_SECOND);
-    let [delta_day, delta_second, delta_nanosecond] = t::NoUnits::vary_many(
-        [day, second, nanosecond],
-        |[day, _second, nanosecond]| {
-            if day >= 0 || nanosecond == 0 {
-                [C(0), C(0), C(0)]
-            } else {
-                [
-                    C(1),
-                    -(t::SECONDS_PER_CIVIL_DAY - C(1)),
-                    (-t::NANOS_PER_SECOND).rinto(),
-                ]
-            }
-        },
-    );
-
-    let day = day + delta_day;
-    let second = day * t::SECONDS_PER_CIVIL_DAY + second + delta_second;
-    // This can overflow our Timestamp limits, so we need to make sure we're
-    // doing arithmetic using `UnixSeconds` boundaries. This is precisely the
-    // point at which a DateTime->Timestamp conversion can fail, and is a
-    // result of guaranteeing that all Timestamp->DateTime conversions succeed.
-    let second = t::UnixSeconds::rfrom(second)
-        .try_checked_sub("offset-second", offset.seconds_ranged())
-        .with_context(|| {
-            err!(
-                "converting {dt} with offset {offset} to timestamp overflowed \
-                 (second={second}, nanosecond={nanosecond})",
-            )
-        })?;
-    let nanosecond = nanosecond + delta_nanosecond;
-    Timestamp::new_ranged(second, nanosecond).with_context(|| {
-        err!(
-            "converting {dt} with offset {offset} to timestamp overflowed \
-             (second={second}, nanosecond={nanosecond})",
-        )
-    })
-}
