@@ -1,30 +1,31 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
 use crate::deserialize::DeserializeError;
-use crate::ffi::*;
+use crate::ffi::{PyBytes_AS_STRING, PyBytes_GET_SIZE, PyMemoryView_GET_BUFFER};
 use crate::str::unicode_to_str;
 use crate::typeref::{BYTEARRAY_TYPE, BYTES_TYPE, MEMORYVIEW_TYPE, STR_TYPE};
+use crate::util::isize_to_usize;
 use crate::util::INVALID_STR;
+use core::ffi::c_char;
 use std::borrow::Cow;
-use std::os::raw::c_char;
 
-#[cfg(all(target_arch = "x86_64", not(target_feature = "sse4.2")))]
+#[cfg(all(target_arch = "x86_64", not(target_feature = "avx2")))]
 fn is_valid_utf8(buf: &[u8]) -> bool {
-    if std::is_x86_feature_detected!("sse4.2") {
-        simdutf8::basic::from_utf8(buf).is_ok()
+    if std::is_x86_feature_detected!("avx2") {
+        unsafe { simdutf8::basic::imp::x86::avx2::validate_utf8(buf).is_ok() }
     } else {
         encoding_rs::Encoding::utf8_valid_up_to(buf) == buf.len()
     }
 }
 
-#[cfg(all(target_arch = "x86_64", target_feature = "sse4.2"))]
+#[cfg(all(target_arch = "x86_64", target_feature = "avx2"))]
 fn is_valid_utf8(buf: &[u8]) -> bool {
     simdutf8::basic::from_utf8(buf).is_ok()
 }
 
 #[cfg(target_arch = "aarch64")]
 fn is_valid_utf8(buf: &[u8]) -> bool {
-    simdutf8::basic::from_utf8(buf).is_ok()
+    unsafe { simdutf8::basic::imp::aarch64::neon::validate_utf8(buf).is_ok() }
 }
 
 #[cfg(not(any(target_arch = "x86_64", target_arch = "aarch64")))]
@@ -39,9 +40,9 @@ pub fn read_input_to_buf(
     let buffer: &[u8];
     if is_type!(obj_type_ptr, BYTES_TYPE) {
         buffer = unsafe {
-            std::slice::from_raw_parts(
-                PyBytes_AS_STRING(ptr) as *const u8,
-                PyBytes_GET_SIZE(ptr) as usize,
+            core::slice::from_raw_parts(
+                PyBytes_AS_STRING(ptr).cast::<u8>(),
+                isize_to_usize(PyBytes_GET_SIZE(ptr)),
             )
         };
         if !is_valid_utf8(buffer) {
@@ -53,7 +54,7 @@ pub fn read_input_to_buf(
             return Err(DeserializeError::invalid(Cow::Borrowed(INVALID_STR)));
         }
         let as_str = uni.unwrap();
-        buffer = unsafe { std::slice::from_raw_parts(as_str.as_ptr(), as_str.len()) };
+        buffer = unsafe { core::slice::from_raw_parts(as_str.as_ptr(), as_str.len()) };
     } else if unlikely!(is_type!(obj_type_ptr, MEMORYVIEW_TYPE)) {
         let membuf = unsafe { PyMemoryView_GET_BUFFER(ptr) };
         if unsafe { pyo3_ffi::PyBuffer_IsContiguous(membuf, b'C' as c_char) == 0 } {
@@ -62,16 +63,16 @@ pub fn read_input_to_buf(
             )));
         }
         buffer = unsafe {
-            std::slice::from_raw_parts((*membuf).buf as *const u8, (*membuf).len as usize)
+            core::slice::from_raw_parts((*membuf).buf as *const u8, isize_to_usize((*membuf).len))
         };
         if !is_valid_utf8(buffer) {
             return Err(DeserializeError::invalid(Cow::Borrowed(INVALID_STR)));
         }
     } else if unlikely!(is_type!(obj_type_ptr, BYTEARRAY_TYPE)) {
         buffer = unsafe {
-            std::slice::from_raw_parts(
+            core::slice::from_raw_parts(
                 ffi!(PyByteArray_AsString(ptr)) as *const u8,
-                ffi!(PyByteArray_Size(ptr)) as usize,
+                isize_to_usize(ffi!(PyByteArray_Size(ptr))),
             )
         };
         if !is_valid_utf8(buffer) {
