@@ -8,10 +8,11 @@ use crate::str::unicode_to_str;
 use crate::typeref::{
     DATACLASS_FIELDS_STR, DICT_STR, FIELD_TYPE, FIELD_TYPE_STR, SLOTS_STR, STR_TYPE,
 };
+use crate::util::isize_to_usize;
 
 use serde::ser::{Serialize, SerializeMap, Serializer};
 
-use std::ptr::NonNull;
+use core::ptr::NonNull;
 
 #[repr(transparent)]
 pub struct DataclassGenericSerializer<'a> {
@@ -83,35 +84,40 @@ impl DataclassFastSerializer {
 }
 
 impl Serialize for DataclassFastSerializer {
+    #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        let len = ffi!(Py_SIZE(self.ptr)) as usize;
+        let len = isize_to_usize(ffi!(Py_SIZE(self.ptr)));
         if unlikely!(len == 0) {
             return ZeroDictSerializer::new().serialize(serializer);
         }
         let mut map = serializer.serialize_map(None).unwrap();
-        let mut next_key: *mut pyo3_ffi::PyObject = std::ptr::null_mut();
-        let mut next_value: *mut pyo3_ffi::PyObject = std::ptr::null_mut();
 
         let mut pos = 0;
+        let mut next_key: *mut pyo3_ffi::PyObject = core::ptr::null_mut();
+        let mut next_value: *mut pyo3_ffi::PyObject = core::ptr::null_mut();
 
         pydict_next!(self.ptr, &mut pos, &mut next_key, &mut next_value);
-        for _ in 0..=ffi!(Py_SIZE(self.ptr)) as usize - 1 {
+
+        for _ in 0..len {
             let key = next_key;
             let value = next_value;
 
             pydict_next!(self.ptr, &mut pos, &mut next_key, &mut next_value);
 
-            if unlikely!(unsafe { ob_type!(key) != STR_TYPE }) {
-                err!(SerializeError::KeyMustBeStr)
-            }
-            let data = unicode_to_str(key);
-            if unlikely!(data.is_none()) {
-                err!(SerializeError::InvalidStr)
-            }
-            let key_as_str = data.unwrap();
+            let key_as_str = {
+                let key_ob_type = ob_type!(key);
+                if unlikely!(!is_class_by_type!(key_ob_type, STR_TYPE)) {
+                    err!(SerializeError::KeyMustBeStr)
+                }
+                let tmp = unicode_to_str(key);
+                if unlikely!(tmp.is_none()) {
+                    err!(SerializeError::InvalidStr)
+                }
+                tmp.unwrap()
+            };
             if unlikely!(key_as_str.as_bytes()[0] == b'_') {
                 continue;
             }
@@ -144,6 +150,8 @@ impl DataclassFallbackSerializer {
 }
 
 impl Serialize for DataclassFallbackSerializer {
+    #[cold]
+    #[inline(never)]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -151,19 +159,19 @@ impl Serialize for DataclassFallbackSerializer {
         let fields = ffi!(PyObject_GetAttr(self.ptr, DATACLASS_FIELDS_STR));
         debug_assert!(ffi!(Py_REFCNT(fields)) >= 2);
         ffi!(Py_DECREF(fields));
-        let len = ffi!(Py_SIZE(fields)) as usize;
+        let len = isize_to_usize(ffi!(Py_SIZE(fields)));
         if unlikely!(len == 0) {
             return ZeroDictSerializer::new().serialize(serializer);
         }
         let mut map = serializer.serialize_map(None).unwrap();
 
-        let mut next_key: *mut pyo3_ffi::PyObject = std::ptr::null_mut();
-        let mut next_value: *mut pyo3_ffi::PyObject = std::ptr::null_mut();
-
         let mut pos = 0;
+        let mut next_key: *mut pyo3_ffi::PyObject = core::ptr::null_mut();
+        let mut next_value: *mut pyo3_ffi::PyObject = core::ptr::null_mut();
 
         pydict_next!(fields, &mut pos, &mut next_key, &mut next_value);
-        for _ in 0..=ffi!(Py_SIZE(fields)) as usize - 1 {
+
+        for _ in 0..len {
             let attr = next_key;
             let field = next_value;
 
@@ -172,14 +180,17 @@ impl Serialize for DataclassFallbackSerializer {
             let field_type = ffi!(PyObject_GetAttr(field, FIELD_TYPE_STR));
             debug_assert!(ffi!(Py_REFCNT(field_type)) >= 2);
             ffi!(Py_DECREF(field_type));
-            if unsafe { field_type as *mut pyo3_ffi::PyTypeObject != FIELD_TYPE } {
+            if unsafe { field_type.cast::<pyo3_ffi::PyTypeObject>() != FIELD_TYPE } {
                 continue;
             }
-            let data = unicode_to_str(attr);
-            if unlikely!(data.is_none()) {
-                err!(SerializeError::InvalidStr);
-            }
-            let key_as_str = data.unwrap();
+
+            let key_as_str = {
+                let tmp = unicode_to_str(attr);
+                if unlikely!(tmp.is_none()) {
+                    err!(SerializeError::InvalidStr)
+                }
+                tmp.unwrap()
+            };
             if key_as_str.as_bytes()[0] == b'_' {
                 continue;
             }
@@ -190,7 +201,7 @@ impl Serialize for DataclassFallbackSerializer {
             let pyvalue = PyObjectSerializer::new(value, self.state, self.default);
 
             map.serialize_key(key_as_str).unwrap();
-            map.serialize_value(&pyvalue)?
+            map.serialize_value(&pyvalue)?;
         }
         map.end()
     }
