@@ -1,19 +1,21 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
-use crate::opt::*;
+use crate::opt::{NON_STR_KEYS, NOT_PASSTHROUGH, SORT_KEYS, SORT_OR_NON_STR_KEYS};
+use crate::serialize::buffer::SmallFixedBuffer;
 use crate::serialize::error::SerializeError;
 use crate::serialize::obtype::{pyobject_to_obtype, ObType};
 use crate::serialize::per_type::datetimelike::DateTimeLike;
 use crate::serialize::per_type::{
-    BoolSerializer, DataclassGenericSerializer, Date, DateTime, DateTimeBuffer, DefaultSerializer,
-    EnumSerializer, FloatSerializer, FragmentSerializer, Int53Serializer, IntSerializer,
-    ListTupleSerializer, NoneSerializer, NumpyScalar, NumpySerializer, StrSerializer,
-    StrSubclassSerializer, Time, ZeroListSerializer, UUID,
+    BoolSerializer, DataclassGenericSerializer, Date, DateTime, DefaultSerializer, EnumSerializer,
+    FloatSerializer, FragmentSerializer, IntSerializer, ListTupleSerializer, NoneSerializer,
+    NumpyScalar, NumpySerializer, StrSerializer, StrSubclassSerializer, Time, ZeroListSerializer,
+    UUID,
 };
 use crate::serialize::serializer::PyObjectSerializer;
 use crate::serialize::state::SerializerState;
 use crate::str::{unicode_to_str, unicode_to_str_via_ffi};
 use crate::typeref::{STR_TYPE, TRUE, VALUE_STR};
+use crate::util::isize_to_usize;
 use compact_str::CompactString;
 use core::ptr::NonNull;
 use serde::ser::{Serialize, SerializeMap, Serializer};
@@ -72,16 +74,17 @@ impl Serialize for DictGenericSerializer {
             ZeroDictSerializer::new().serialize(serializer)
         } else if likely!(opt_disabled!(self.state.opts(), SORT_OR_NON_STR_KEYS)) {
             unsafe {
-                core::mem::transmute::<&DictGenericSerializer, &Dict>(self).serialize(serializer)
+                (*(core::ptr::from_ref::<DictGenericSerializer>(self)).cast::<Dict>())
+                    .serialize(serializer)
             }
         } else if opt_enabled!(self.state.opts(), NON_STR_KEYS) {
             unsafe {
-                core::mem::transmute::<&DictGenericSerializer, &DictNonStrKey>(self)
+                (*(core::ptr::from_ref::<DictGenericSerializer>(self)).cast::<DictNonStrKey>())
                     .serialize(serializer)
             }
         } else {
             unsafe {
-                core::mem::transmute::<&DictGenericSerializer, &DictSortedKey>(self)
+                (*(core::ptr::from_ref::<DictGenericSerializer>(self)).cast::<DictSortedKey>())
                     .serialize(serializer)
             }
         }
@@ -100,13 +103,8 @@ macro_rules! impl_serialize_entry {
                 $map.serialize_value(&StrSubclassSerializer::new($value))?;
             }
             ObType::Int => {
-                if unlikely!(opt_enabled!($self.state.opts(), STRICT_INTEGER)) {
-                    $map.serialize_key($key).unwrap();
-                    $map.serialize_value(&Int53Serializer::new($value))?;
-                } else {
-                    $map.serialize_key($key).unwrap();
-                    $map.serialize_value(&IntSerializer::new($value))?;
-                }
+                $map.serialize_key($key).unwrap();
+                $map.serialize_value(&IntSerializer::new($value, $self.state.opts()))?;
             }
             ObType::None => {
                 $map.serialize_key($key).unwrap();
@@ -227,7 +225,7 @@ impl Serialize for Dict {
 
         let mut map = serializer.serialize_map(None).unwrap();
 
-        let len = ffi!(Py_SIZE(self.ptr)) as usize;
+        let len = isize_to_usize(ffi!(Py_SIZE(self.ptr)));
         assume!(len > 0);
 
         for _ in 0..len {
@@ -245,7 +243,7 @@ impl Serialize for Dict {
                 let tmp = unicode_to_str(key);
                 if unlikely!(tmp.is_none()) {
                     err!(SerializeError::InvalidStr)
-                };
+                }
                 tmp.unwrap()
             };
 
@@ -275,7 +273,7 @@ impl Serialize for DictSortedKey {
 
         pydict_next!(self.ptr, &mut pos, &mut next_key, &mut next_value);
 
-        let len = ffi!(Py_SIZE(self.ptr)) as usize;
+        let len = isize_to_usize(ffi!(Py_SIZE(self.ptr)));
         assume!(len > 0);
 
         let mut items: SmallVec<[(&str, *mut pyo3_ffi::PyObject); 8]> =
@@ -331,9 +329,10 @@ fn non_str_str_subclass(key: *mut pyo3_ffi::PyObject) -> Result<CompactString, S
     }
 }
 
+#[allow(clippy::unnecessary_wraps)]
 #[inline(never)]
 fn non_str_date(key: *mut pyo3_ffi::PyObject) -> Result<CompactString, SerializeError> {
-    let mut buf = DateTimeBuffer::new();
+    let mut buf = SmallFixedBuffer::new();
     Date::new(key).write_buf(&mut buf);
     let key_as_str = str_from_slice!(buf.as_ptr(), buf.len());
     Ok(CompactString::from(key_as_str))
@@ -344,7 +343,7 @@ fn non_str_datetime(
     key: *mut pyo3_ffi::PyObject,
     opts: crate::opt::Opt,
 ) -> Result<CompactString, SerializeError> {
-    let mut buf = DateTimeBuffer::new();
+    let mut buf = SmallFixedBuffer::new();
     let dt = DateTime::new(key, opts);
     if dt.write_buf(&mut buf, opts).is_err() {
         return Err(SerializeError::DatetimeLibraryUnsupported);
@@ -359,7 +358,7 @@ fn non_str_time(
     key: *mut pyo3_ffi::PyObject,
     opts: crate::opt::Opt,
 ) -> Result<CompactString, SerializeError> {
-    let mut buf = DateTimeBuffer::new();
+    let mut buf = SmallFixedBuffer::new();
     let time = Time::new(key, opts);
     if time.write_buf(&mut buf).is_err() {
         return Err(SerializeError::TimeHasTzinfo);
@@ -368,25 +367,28 @@ fn non_str_time(
     Ok(CompactString::from(key_as_str))
 }
 
+#[allow(clippy::unnecessary_wraps)]
 #[inline(never)]
 fn non_str_uuid(key: *mut pyo3_ffi::PyObject) -> Result<CompactString, SerializeError> {
-    let mut buf = arrayvec::ArrayVec::<u8, 36>::new();
+    let mut buf = SmallFixedBuffer::new();
     UUID::new(key).write_buf(&mut buf);
     let key_as_str = str_from_slice!(buf.as_ptr(), buf.len());
     Ok(CompactString::from(key_as_str))
 }
 
+#[allow(clippy::unnecessary_wraps)]
 #[cold]
 #[inline(never)]
 fn non_str_float(key: *mut pyo3_ffi::PyObject) -> Result<CompactString, SerializeError> {
     let val = ffi!(PyFloat_AS_DOUBLE(key));
     if !val.is_finite() {
-        Ok(CompactString::new_inline("null"))
+        Ok(CompactString::const_new("null"))
     } else {
         Ok(CompactString::from(ryu::Buffer::new().format_finite(val)))
     }
 }
 
+#[allow(clippy::unnecessary_wraps)]
 #[inline(never)]
 fn non_str_int(key: *mut pyo3_ffi::PyObject) -> Result<CompactString, SerializeError> {
     let ival = ffi!(PyLong_AsLongLong(key));
@@ -419,12 +421,12 @@ impl DictNonStrKey {
         opts: crate::opt::Opt,
     ) -> Result<CompactString, SerializeError> {
         match pyobject_to_obtype(key, opts) {
-            ObType::None => Ok(CompactString::new_inline("null")),
+            ObType::None => Ok(CompactString::const_new("null")),
             ObType::Bool => {
                 if unsafe { key == TRUE } {
-                    Ok(CompactString::new_inline("true"))
+                    Ok(CompactString::const_new("true"))
                 } else {
-                    Ok(CompactString::new_inline("false"))
+                    Ok(CompactString::const_new("false"))
                 }
             }
             ObType::Int => non_str_int(key),
@@ -468,7 +470,7 @@ impl Serialize for DictNonStrKey {
 
         let opts = self.state.opts() & NOT_PASSTHROUGH;
 
-        let len = ffi!(Py_SIZE(self.ptr)) as usize;
+        let len = isize_to_usize(ffi!(Py_SIZE(self.ptr)));
         assume!(len > 0);
 
         let mut items: SmallVec<[(CompactString, *mut pyo3_ffi::PyObject); 8]> =
