@@ -2,7 +2,6 @@
 // This is an adaptation of `src/value/ser.rs` from serde-json.
 
 use crate::serialize::writer::formatter::{CompactFormatter, Formatter, PrettyFormatter};
-use crate::serialize::writer::str::*;
 use crate::serialize::writer::WriteExt;
 use serde::ser::{self, Impossible, Serialize};
 use serde_json::error::{Error, Result};
@@ -581,7 +580,7 @@ where
     unsafe {
         reserve_str!(writer, value);
 
-        let written = format_escaped_str_impl_generic_128(
+        let written = crate::serialize::writer::str::format_escaped_str_impl_generic_128(
             writer.as_mut_buffer_ptr(),
             value.as_bytes().as_ptr(),
             value.len(),
@@ -591,6 +590,22 @@ where
     }
 }
 
+#[cfg(all(feature = "unstable-simd", target_arch = "x86_64", feature = "avx512"))]
+type StrFormatter = unsafe fn(*mut u8, *const u8, usize) -> usize;
+
+#[cfg(all(feature = "unstable-simd", target_arch = "x86_64", feature = "avx512"))]
+static mut STR_FORMATTER_FN: StrFormatter =
+    crate::serialize::writer::str::format_escaped_str_impl_sse2_128;
+
+pub fn set_str_formatter_fn() {
+    unsafe {
+        #[cfg(all(feature = "unstable-simd", target_arch = "x86_64", feature = "avx512"))]
+        if std::is_x86_feature_detected!("avx512vl") {
+            STR_FORMATTER_FN = crate::serialize::writer::str::format_escaped_str_impl_512vl;
+        }
+    }
+}
+
 #[cfg(all(
     feature = "unstable-simd",
     target_arch = "x86_64",
@@ -604,7 +619,7 @@ where
     unsafe {
         reserve_str!(writer, value);
 
-        let written = format_escaped_str_impl_sse2_128(
+        let written = crate::serialize::writer::str::format_escaped_str_impl_sse2_128(
             writer.as_mut_buffer_ptr(),
             value.as_bytes().as_ptr(),
             value.len(),
@@ -623,21 +638,13 @@ where
     unsafe {
         reserve_str!(writer, value);
 
-        if std::is_x86_feature_detected!("avx512vl") {
-            let written = format_escaped_str_impl_512vl(
-                writer.as_mut_buffer_ptr(),
-                value.as_bytes().as_ptr(),
-                value.len(),
-            );
-            writer.set_written(written);
-        } else {
-            let written = format_escaped_str_impl_sse2_128(
-                writer.as_mut_buffer_ptr(),
-                value.as_bytes().as_ptr(),
-                value.len(),
-            );
-            writer.set_written(written);
-        };
+        let written = STR_FORMATTER_FN(
+            writer.as_mut_buffer_ptr(),
+            value.as_bytes().as_ptr(),
+            value.len(),
+        );
+
+        writer.set_written(written);
     }
 }
 
@@ -650,11 +657,12 @@ where
     unsafe {
         reserve_str!(writer, value);
 
-        let written = format_escaped_str_scalar(
+        let written = crate::serialize::writer::str::format_escaped_str_scalar(
             writer.as_mut_buffer_ptr(),
             value.as_bytes().as_ptr(),
             value.len(),
         );
+
         writer.set_written(written);
     }
 }
@@ -668,7 +676,7 @@ where
     unsafe {
         reserve_str!(writer, value);
 
-        let written = format_escaped_str_impl_sse2_128(
+        let written = crate::serialize::writer::str::format_escaped_str_impl_sse2_128(
             writer.as_mut_buffer_ptr(),
             value.as_bytes().as_ptr(),
             value.len(),
