@@ -49,7 +49,7 @@ class TestApi:
         """
         loads() invalid type
         """
-        for val in (1, 3.14, [], {}, None):
+        for val in (1, 3.14, [], {}, None):  # type: ignore
             pytest.raises(orjson.JSONDecodeError, orjson.loads, val)
 
     def test_loads_recursion_partial(self):
@@ -79,7 +79,7 @@ class TestApi:
         loads() recursion limit at limit mixed
         """
         n = LOADS_RECURSION_LIMIT
-        value = b"[" b'{"key":' * n + b'{"key":true}' + b"}" * n + b"]"
+        value = b"".join((b"[", b'{"key":' * n, b'{"key":true}' + b"}" * n, b"]"))
         pytest.raises(orjson.JSONDecodeError, orjson.loads, value)
 
     def test_loads_recursion_valid_excessive_array(self):
@@ -111,7 +111,7 @@ class TestApi:
         loads() recursion limit at limit mixed pretty
         """
         n = LOADS_RECURSION_LIMIT
-        value = b"[\n  " b'{"key":' * n + b'{"key":true}' + b"}" * n + b"]"
+        value = b'[\n  {"key":' * n + b'{"key":true}' + b"}" * n + b"]"
         pytest.raises(orjson.JSONDecodeError, orjson.loads, value)
 
     def test_loads_recursion_valid_excessive_array_pretty(self):
